#include "system.h"

/* This will keep track of how many ticks that the system
*  has been running for */
unsigned int timer_ticks = 0;

/* Handles the timer. In this case, it's very simple: We
*  increment the 'timer_ticks' variable every time the
*  timer fires. By default, the timer fires 18.222 times
*  per second. Why 18.222Hz? Some engineer at IBM must've
*  been smoking something funky */

void timer_phase(uint16 hz)
{
    uint16 divisor = 1193180 / hz;       /* Calculate our divisor */
    outportb(0x43, 0x36);             /* Set our command byte 0x36 */
    outportb(0x40, divisor & 0xFF);   /* Set low byte of divisor */
    outportb(0x40, divisor >> 8);     /* Set high byte of divisor */
}


void timer_handler(struct regs *r)
{
    /* Increment our 'tick count' */
    timer_ticks++;

    /* Every 18 clocks (approximately 1 second), we will
    *  display a message on the screen */
    if (timer_ticks % 100 == 0)
    {
        color a(255,255,255),b(129,129,129);
        System.print("Timer: ",0,24,a,b);
        System.printi(timer_ticks/100,10,strlen("Timer: "),24,a,b);

    }
        int1c();
}

/* Sets up the system clock by installing the timer handler
*  into IRQ0 */
void timer_install()
{
     timer_ticks=0;         //Some computers has problems without this
    /* Installs 'timer_handler' to IRQ0 */
    timer_phase(100);
    irq_install_handler(0, timer_handler);
}
	
void timer_wait(int ticks)
{
    unsigned long eticks;

    eticks = timer_ticks + ticks;
    while(timer_ticks < eticks);
}


void spk_play(uint32 nFrequence) {
 	uint32 Div;
 	uint8 tmp;
 
 	Div = 1193180 / nFrequence;
 	outportb(0x43, 0xb6);
 	outportb(0x42, (uint8) (Div) );
 	outportb(0x42, (uint8) (Div >> 8));
 
 	tmp = inportb(0x61);
  	if (tmp != (tmp | 3)) {
 		outportb(0x61, tmp | 3);
 	}
}
 
void spk_nosound() {
 	uint8 tmp = (inportb(0x61) & 0xFC);
 
 	outportb(0x61, tmp);
 }
 
void spk_beep() {
 	 spk_play(1000);
 	 timer_wait(10);
 	 spk_nosound();
}
